from __future__ import unicode_literals

from unittest import TestCase
from collections import OrderedDict
from xamcheck_utils.importlib import import_from_string


class TestUtilsimportlib(TestCase):

    def test_import_from_string(self):
        items = (
            ('collections.OrderedDict', OrderedDict),
            ('unittest.TestCase', TestCase),
        )
        for value, output in items:

            self.assertEqual(import_from_string(value), output)
        items = (
            ('colections_foobar_mo_random.OderedDict'),
        )
        for value in items:
            with self.assertRaises(ImportError):
                import_from_string(value)

        items = (
            ('collections.OderedDic_random_thing_from_cloudt'),
        )
        for value in items:
            with self.assertRaises(AttributeError):
                import_from_string(value)



from __future__ import unicode_literals
from django.core.exceptions import ValidationError

from unittest import TestCase
from xamcheck_utils import excel
from os.path import abspath, dirname, join, normpath

CURRENT_DIR = dirname(abspath(__file__))
EXCEL_FILES_ROOT = normpath(join(CURRENT_DIR, 'data/excel-files'))

excel_file_path = normpath(join(EXCEL_FILES_ROOT, "excel1.xlsx"))

with open(excel_file_path, 'rb') as excel_file:
    excel_file_contents = excel_file.read()
    workbook = excel.get_workbook(excel_file_contents)


class TestUtilsExcelUtils(TestCase):

    def test_get_workbook(self):
        workbook = excel.get_workbook(excel_file_contents)
        self.assertEqual(workbook.sheet_names(), [u'Sheet1'])

    def test_get_worksheet_by_index(self):
        worksheet = excel.get_worksheet_by_index(workbook, 0)
        self.assertEqual(worksheet.columns_from_right_to_left, 0)

    def test_get_worksheet_by_name(self):
        worksheet = excel.get_worksheet_by_name(workbook, 'Sheet1')
        self.assertEqual(worksheet.columns_from_right_to_left, 0)

    def test_validate_worksheet_size(self):
        worksheet = workbook.sheet_by_index(0)
        items = (
            (worksheet, 0, 3),
            (worksheet, 3, 0),
        )
        for (worksheet, min_cols, min_rows) in items:
            with self.assertRaises(ValidationError):
                excel.validate_worksheet_size(
                    worksheet, min_cols, min_rows)

    def test_get_worksheet_by_name_from_file(self):
        worksheet = excel.get_worksheet_by_name_from_file(
            excel_file_contents, 'Sheet1')
        self.assertEqual(worksheet.columns_from_right_to_left, 0)
        items = (
            (excel_file_contents, 'Sheet1', 1, 1),
        )
        for (workbook_file, sheet_name, min_cols, min_rows) in items:
            with self.assertRaises(ValidationError):
                excel.get_worksheet_by_name_from_file(
                    workbook_file, sheet_name, min_cols, min_rows)

    def test_get_first_worksheet(self):
        worksheet = excel.get_first_worksheet(
            excel_file_contents, 0, 0)
        self.assertEqual(worksheet.columns_from_right_to_left, 0)
        items = (
            (excel_file_contents, 1, 1),
        )
        for (workbook_file, min_cols, min_rows) in items:
            with self.assertRaises(ValidationError):
                excel.get_first_worksheet(
                    workbook_file, min_cols, min_rows)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import itertools

import re

from django.utils.html import escape


def unicodify(s, strip=True):
    """ Encodes given string to unicode
    >>> unicodify(u'\u00c3')
    '\\xc3\\x83'
    """

    s = escape(s)
    if not (isinstance(s, str) or isinstance(s, unicode)):
        return s
    if isinstance(s, unicode):
        s = s.encode('utf8')
    if strip:
        return s.strip()
    else:
        return s


def remove_spaces(s):
    """ Removes spaces from a given string using regular expressions

    here is an extra line
    >>> remove_spaces("   foo  \\n bar \\t moo")
    u'foobarmoo'

    yet another line then we will add a doctest

    >>> remove_spaces("foo lo bar         ")
    u'foolobar'
    """
    return ''.join(re.split(r'\s', s))


def fix_name(name):
    """ Removes unwanted spaces in a name
    >>> fix_name("hello-world")
    u'hello - world'
    >>> fix_name("hello          world")
    u'hello world'
    """
    name = re.sub("\s*-\s*", " - ", name)
    name = re.sub("\s+", " ", name)
    return name


def fix_student_name(name):
    """ Fix a student's name
    >>> fix_student_name("p v r reddy")
    u'P. V. R. Reddy'
    """
    name = fix_name(name)
    parts = name.split()
    new_parts = []

    for part in parts:
        if len(part) == 1:
            part = part + '.'
        new_parts.append(part.title())

    name = ' '.join(new_parts)
    name = name
    return name


def sorted_nicely(l):
    """Sort the given list in the way that humans expect.
    >>> sorted_nicely(["Q1", "Q3", "Q2", "Q20", "Q11"])
    [u'Q1', u'Q2', u'Q3', u'Q11', u'Q20']
    """
    l = list(l)

    convert = lambda text: int(text) if text.isdigit() else text
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return sorted(l, key=alphanum_key)



import re
from django.utils.html import escape


def unicodify(s, strip=True):  # Testcases not written.
    s = escape(s)
    if not (isinstance(s, str) or isinstance(s, unicode)):
        return s
    if isinstance(s, unicode):
        s = s.encode('utf8')
    if strip:
        return s.strip()
    else:
        return s


def number2string(s):
    """ Return string representation of a number.

    Returns as it is, if ValueError is raised.    #this feature not working
    Returns string exponent form for very large numbers.

    >>> number2string(21.3) == '21.3'
    True
    >>> number2string(21) == '21'
    True
    >>> number2string('lol') == 'lol'
    True
    """
    try:
        return '{0:g}'.format(float(s))
    except ValueError:
        return s


def string2float(s):
    """ Returns float value if number passed as string.
    >>> string2float('111.111') == 111.111
    True

    """
    return float(s)


def string2int(s):
    """ Returns integer value if number is passed as string.
    >>> string2int('111.111') == 111
    True
    """
    return int(float(s))

"""Comma seperated strings to list conversion"""


# Don't know exactly what these 2 lines do
REGEX_COMMA_SEPARATED_INT = re.compile(r"^,|,$|\s")
REGEXT_COMMA = re.compile(r",")


def comma_separated_int_to_list(s):
    """ Returns list containing integers if numbers are 
        passed as string seperated by commas
    >>> comma_separated_int_to_list('111,12') == [111, 12]
    True
    """
    s = REGEX_COMMA_SEPARATED_INT.sub("", s)
    return map(string2int, filter(len, REGEXT_COMMA.split(s)))


def comma_separated_float_to_list(s):
    """ Returns a list containing float values of the 
        numbers passes as string seperated by commas.

    >>> comma_separated_float_to_list('11.1,12') == [11.1, 12.0]
    True
    >>> comma_separated_float_to_list('11,12.1,0.4') == [11.0, 12.1, 0.4]
    True
    """
    s = REGEX_COMMA_SEPARATED_INT.sub("", s)
    return map(string2float, filter(len, REGEXT_COMMA.split(s)))


def comma_separated_str_to_list(s):
    """Separetes string based on commas and stores each 
        string as different strings in a list.

    >>> comma_separated_str_to_list('Ram,Laxman') == ['Ram', 'Laxman']
    True
    >>> comma_separated_str_to_list('Ram,laxman') == ['Ram', 'laxman']
    True
    >>> comma_separated_str_to_list('111,112') == ['111', '112']
    True
    """

    s = REGEX_COMMA_SEPARATED_INT.sub("", s)
    return map(unicodify, filter(len, REGEXT_COMMA.split(s)))


def percent(a, b, r=2):
    try:
        return round(float(a) / float(b) * 100, r)
    except:
        return 0



import re

from html2text import HTML2Text


def check_if_html_has_text(html):
    """ Checks if given html string has any text
    >>> check_if_html_has_text("<p>   &nbsp;</p>")
    False
    >>> check_if_html_has_text("<p>   &  \\n</p>")
    True
    """
    html = html.encode('ascii', 'xmlcharrefreplace')
    # html = html.decode('utf-8', 'ignore')
    h = HTML2Text()
    h.ul_item_mark = ''
    h.emphasis_mark = ''
    h.strong_mark = ''

    text = h.handle(html)

    text = re.sub(r"\s", "", text)

    if text == "":
        return False
    return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xamcheck_utils import text

from .base import TestCase


class TestUtilsText(TestCase):

    def test_unicodify(self):
        f = text.unicodify
        items = (
            ('&', '&amp;'),
            ('<', '&lt;'),
            ('>', '&gt;'),
            ('"', '&quot;'),
            ("'", '&#39;'),
        )

        for value, output in items:
            self.check_output(f, output, value)

    def test_remove_spaces(self):
        f = text.remove_spaces
        items = (
            ('new word', 'newword'),
            ('new word ', 'newword'),
            ('new word\n\rlol', 'newwordlol'),
        )

        for value, output in items:
            self.check_output(f, output, value)

    def test_fix_student_name(self):
        f = text.fix_student_name
        items = (
            ("p v r reddy", "P. V. R. Reddy"),
            ("p v r redd", "P. V. R. Redd"),
            ("pankaj    singh", "Pankaj Singh"),
            ("ram\n chandra", "Ram Chandra"),
        )

        for value, output in items:
            self.check_output(f, output, value)

    def test_soretd_nicely(self):
        """
        Checking whether the function sorted_nicely works properly
        for different types of test cases
        """

        f = text.sorted_nicely
        items = (
            (['1', '11', '2', '3', '21'], ['1', '2', '3', '11', '21']),
        )

        for value, output in items:
            self.check_output(f, output, value)

    def test_fix_name(self):
        f = text.fix_name
        items = (
            ("Mahatma    Gandhi", "Mahatma Gandhi"),
            ("Air-base", "Air - base"),
        )

        for value, output in items:
            self.check_output(f, output, value)



import re
import itertools


def percent(a, b, r=2):
    try:
        return round(float(a) / float(b) * 100, r)
    except:
        return 0


def to_alphanumneric_str(s):
    return "".join(re.findall("[a-zA-Z0-9]+", s)).lower()


def drange(start, stop, step, include_stop=False):
    r = start
    if include_stop is True:
        while r <= stop:
            yield r
            r += step
    else:
        while r < stop:
            yield r
            r += step



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .base import TestCase
from xamcheck_utils.collections import DefaultOrderedDict, OrderedSet


class TestOrderedSet(TestCase):

    def test_add(self):
        ordered_set = OrderedSet()

        ordered_set.add("something")
        self.assertEqual(len(ordered_set), 1)

        ordered_set.add("something")
        self.assertEqual(len(ordered_set), 1)

        ordered_set.add(1)
        self.assertEqual(len(ordered_set), 2)

        new_obj = object()
        ordered_set.add(new_obj)
        self.assertEqual(len(ordered_set), 3)

        another_object = object()
        ordered_set.add(another_object)
        self.assertEqual(len(ordered_set), 4)

        ordered_set.add(another_object)
        self.assertEqual(len(ordered_set), 4)

    def test_discard(self):
        new_obj = object()
        another_object = object()
        another_objectx = another_object
        ordered_set = OrderedSet(['key', 1, '123', new_obj])
        ordered_set.add(another_object)

        ordered_set.discard(another_objectx)
        self.assertEqual(len(ordered_set), 4)

        ordered_set.discard(1)
        self.assertEqual(len(ordered_set), 3)

        ordered_set.discard(another_object)
        self.assertEqual(len(ordered_set), 3)

        ordered_set.discard(new_obj)
        self.assertEqual(len(ordered_set), 2)

        ordered_set.discard('123')
        self.assertEqual(len(ordered_set), 1)

        ordered_set.discard('KEY')
        self.assertEqual(len(ordered_set), 1)

        ordered_set.discard('key')
        self.assertEqual(len(ordered_set), 0)

    def test_equal(self):
        new_obj = object()
        another_object = object()
        ordered_set = OrderedSet(['key', 1, '123', new_obj, another_object])
        another_ordered_set = OrderedSet(['key', 1, '123', new_obj])
        other_ordered_set = OrderedSet(
            ['key', 1, '123', new_obj, another_object])

        ordered_set.__eq__(other_ordered_set)
        self.assertEqual(set(ordered_set), set(other_ordered_set))

        ordered_set.__eq__(another_ordered_set)
        set(ordered_set) != set(another_ordered_set)

    def test_pop(self):
        new_obj = object()
        ordered_set = OrderedSet(['key', 1, '123', new_obj])

        ordered_set.pop()
        self.assertEqual(len(ordered_set), 3)

        ordered_set.pop()
        self.assertEqual(len(ordered_set), 2)

        ordered_set.pop()
        self.assertEqual(len(ordered_set), 1)

        ordered_set.pop()
        self.assertEqual(len(ordered_set), 0)

        with self.assertRaises(KeyError):
            ordered_set.pop()

    def test_contains(self):

        new_obj = object()
        ordered_set = OrderedSet(['key', 1, '123', new_obj])
        items = (
            ('key', True),
            ('2', False),
            ('123', True),
            (2, False),
            (new_obj, True),

        )

        for value, output in items:
            self.check_output(ordered_set.__contains__, output, value)


class TestOrderedDict(TestCase):

    def test_getitem(self):
        items = [('yellow', 1), ('blue', 2),
                 ('yellow', 3), ('blue', 4), ('red', 1), (1, 'samsung')]
        default_ordered_dict = DefaultOrderedDict(list)
        for k, v in items:
            default_ordered_dict[k].append(v)

        values = default_ordered_dict.__getitem__('yellow')
        self.assertEqual(values, [1, 3])

        values = default_ordered_dict.__getitem__(1)
        self.assertEqual(values, ['samsung'])

        values = default_ordered_dict.__getitem__('lenovo')
        self.assertEqual(values, [])

    # def test_missing(self):
    #     items = [('yellow', 1), ('blue', 2), ('yellow', 3),
    #              ('blue', 4), ('red', 1), (1, 'samsung')]
    #     default_ordered_dict = DefaultOrderedDict(list)
    #     for k, v in items:
    #         default_ordered_dict[k].append(v)

    #     values = default_ordered_dict.__getitem__('titan')
    #     self.assertEqual(values, [])



from __future__ import absolute_import

from django.utils.importlib import import_module


def import_from_string(s):
    """ Imports the method specified after the last period in
        the string.
    >>> from collections import deque
    >>> import_from_string('collections.deque') == deque
    True
    """
    # grab the classname/function name off of the string
    package, attrib = s.rsplit('.', 1)

    # dynamically import the module
    module = import_module(package)

    # pull the class/function off the module and return
    return getattr(module, attrib)



from __future__ import unicode_literals

from unittest import TestCase

from xamcheck_utils import conversions


class TestUtilsText(TestCase):

    def check_output(self, function, output=None, *value):
        """
        Check that function(value) equals output.  If output is None,
        check that function(value) equals value.
        """
        if output is None:
            output = None
        self.assertEqual(function(*value), output)

    def test_number2string(self):
        f = conversions.number2string
        items = (
            ('1.11116e+22', '11111555555555555555555'),
            ('11', '11.0'),
            ('0.1', '0.1')
        )

        for output, value in items:
            self.check_output(f, output, value)

    def test_string2float(self):
        f = conversions.string2float
        items = (
            (11.0, "11"),
            (11.11, "11.11"),
        )

        for output, value in items:
            self.check_output(f, output, value)

    def test_string2int(self):
        f = conversions.string2int
        items = (
            (0, '0.12'),

        )

        for output, value in items:
            self.check_output(f, output, value)

    """Now testing comma seperated string to lists conversion"""

    def test_comma_separated_int_to_list(self):
        f = conversions.comma_separated_int_to_list
        items = (
            ([11, 12, 13], '11,12.4,13.5'),
            ([11, 0], '11.3,0.5'),
        )

        for value, output in items:
            self.check_output(f, value, output)

    def test_comma_separated_float_to_list(self):
        """Tests performed in doctests probably no
            other testcases required
        """
        pass

    def test_comma_separated_str_to_list(self):
        """ Tests performed in doctests probably no
            other test cases required.
        """
        pass

    def test_percent(self):
        f = conversions.percent
        items = (
            (1.0, 1, 100),
            (0, 1, 'a'),
        )

        for output, value1, value2 in items:
            self.check_output(f, output, value1, value2)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xamcheck_utils import func

from .base import TestCase


class TestUtilsFunc(TestCase):

    def test_percent(self):
        f = func.percent
        items = (
            (12, 'a', 0),
            (30, '110', 27.27),
            (30, 110, 27.27),

        )

        for value1, value2, output in items:
            self.check_output(f, output, value1, value2)

    def test_to_alphanumneric_str(self):
        f = func.to_alphanumneric_str
        items = (
            ('Python Code', 'pythoncode'),
            ('Python_Code', 'pythoncode'),
            ('Python_Code123', 'pythoncode123'),
            ('Python code', 'pythoncode'),

        )

        for value, output in items:
            self.check_output(f, output, value)

    def test_drange(self):
        f = func.drange
        items = (
            ((1, 2, 0.25), [1, 1.25, 1.5, 1.75]),
        )

        for (start, stop, step), output in items:
            self.assertEqual(list(f(start, stop, step)), output)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from unittest import TestCase as OriginalTestCase


class TestCase(OriginalTestCase):

    def check_output(self, function, output=None, *value):
        """
        Check that function(value) equals output.  If output is None,
        check that function(value) equals value.
        """
        if output is None:
            output = None
        self.assertEqual(function(*value), output)



import os

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


class NoseTestCommand(TestCommand):
    # Inspired by the example at https://pytest.org/latest/goodpractises.html

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # Run nose ensuring that argv simulates running nosetests directly
        import nose
        nose.run_exit(argv=['nosetests'])

TESTS_REQUIRE = [
    'coverage',
    'nose',
]


setup(
    name="xamcheck_utils",
    version="0.0.8",
    author="Pankaj Singh",
    author_email="pankaj@policyinnovations.in",
    description=("Utility functions used in python projects of Xamcheck."),
    license = "BSD",
    keywords = "xamcheck utility",
    url = "https://github.com/psjinx/xamcheck-utils",
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['xamcheck_utils'],
    extras_require=dict(
        test=TESTS_REQUIRE,
    ),
    install_requires=[
        'Django<1.7',
        'setuptools',
    ],
    cmdclass={'test': NoseTestCommand},
    tests_require=TESTS_REQUIRE,
    test_suite = "nose.collector",
    include_package_data=True,
    zip_safe=False,
)



from django.core.exceptions import ValidationError

import xlrd


def get_workbook(workbook_file):
    """Opens the excel file whose name"""
    error_messages = []

    try:
        workbook = xlrd.open_workbook(file_contents=workbook_file)
    except xlrd.XLRDError as e:
        error_messages.append(e.message)
        raise ValidationError(error_messages)

    return workbook


def get_worksheet_by_index(workbook, index=0):
    """Returns worksheet having the specified index"""
    error_messages = []

    try:
        worksheet = workbook.sheet_by_index(index)
    except IndexError as e:
        error_messages.append(e.message)
        raise ValidationError(error_messages)

    return worksheet


def get_worksheet_by_name(workbook, sheet_name):
    """Returns worksheet with the given name
        in the given workbook"""
    error_messages = []

    try:
        worksheet = workbook.sheet_by_name(sheet_name)
    except IndexError as e:
        error_messages.append(e.message)
        raise ValidationError(error_messages)

    return worksheet


def validate_worksheet_size(worksheet, min_cols, min_rows):
    """ Validates the size of the worksheet to verify
        the minimum number of rows and columns
    """
    error_messages = []

    ncols = worksheet.ncols
    nrows = worksheet.nrows

    if ncols < min_cols:
        error_messages.append('Invalid worksheet - atleast {0} columns must '
                              'be present.'.format(min_cols))
        raise ValidationError(error_messages)

    if nrows < min_rows:
        error_messages.append('Invalid worksheet - atleast {0} rows must be '
                              'present.'.format(min_rows))
        raise ValidationError(error_messages)


def get_first_worksheet(workbook_file, min_cols=0, min_rows=0):
    """ Returns the first worksheet of the specified workbook"""
    workbook = get_workbook(workbook_file)
    worksheet = get_worksheet_by_index(workbook, index=0)

    if min_cols > 0 and min_rows > 0:
        validate_worksheet_size(worksheet, min_cols, min_rows)

    return worksheet


def get_worksheet_by_name_from_file(workbook_file, sheet_name, min_cols=0, min_rows=0):
    """ Returns the worksheet with the given name and
        workbook file
    """
    workbook = get_workbook(workbook_file)
    worksheet = get_worksheet_by_name(workbook, sheet_name)

    if min_cols > 0 and min_rows > 0:
        validate_worksheet_size(worksheet, min_cols, min_rows)

    return worksheet



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xamcheck_utils import html

from .base import TestCase


class TestUtilsHTML(TestCase):

    def test_check_if_html_has_text(self):
        f = html.check_if_html_has_text
        items = (
            ('<br/>', False),
            ('<p>lol</p>', True),
            ('<p>&nbsp;</p>', False),
        )

        for value, output in items:
            self.check_output(f, output, value)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xamcheck_utils import crypto

from .base import TestCase


class TestCrypto(TestCase):

    def test_hash_string(self):
        function_crypto = crypto.hash_string
        items = (
            ('samsung', '4c5849d848150bbcbfdd69b5dab68db028c1ce182c7d194e8d4d0cd40052ef6e04a2154eca4fb5a8514acf1714660f9ac186d552f1b33d6ae131b74b3370b2a0'),
            ('12345', '3627909a29c31381a071ec27f7c9ca97726182aed29a7ddd2e54353322cfb30abb9e3a6df2ac2c20fe23436311d678564d0c8d305930575f60e2d3d048184d79'),
            ('samsung12345', '6e50ef68b36c4206d9f122193f4c622faadd1c4ca8874df3a0cc5e4647e63eb7ee2d88752d25b4801f2bc48c7b4e70a6420949f4fe4c145d9afd5f627cb08507'),
        )

        for value, output in items:
            self.check_output(function_crypto, output, value)
